@@ -3,7 +3,7 @@ from .utils import NamespacedClient, query_params, _make_path, SKIP_IN_PATH
 
 class SnapshotClient(NamespacedClient):
     @query_params("master_timeout", "wait_for_completion")
-    def create(self, repository, snapshot, body=None, params=None):
+    def create(self, repository, snapshot, body=None, params=None, headers=None):
         """
         Creates a snapshot in a repository.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -24,11 +24,12 @@ class SnapshotClient(NamespacedClient):
             "PUT",
             _make_path("_snapshot", repository, snapshot),
             params=params,
+            headers=headers,
             body=body,
         )
 
     @query_params("master_timeout")
-    def delete(self, repository, snapshot, params=None):
+    def delete(self, repository, snapshot, params=None, headers=None):
         """
         Deletes a snapshot.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -43,11 +44,14 @@ class SnapshotClient(NamespacedClient):
                 raise ValueError("Empty value passed for a required argument.")
 
         return self.transport.perform_request(
-            "DELETE", _make_path("_snapshot", repository, snapshot), params=params
+            "DELETE",
+            _make_path("_snapshot", repository, snapshot),
+            params=params,
+            headers=headers,
         )
 
     @query_params("ignore_unavailable", "master_timeout", "verbose")
-    def get(self, repository, snapshot, params=None):
+    def get(self, repository, snapshot, params=None, headers=None):
         """
         Returns information about a snapshot.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -67,11 +71,14 @@ class SnapshotClient(NamespacedClient):
                 raise ValueError("Empty value passed for a required argument.")
 
         return self.transport.perform_request(
-            "GET", _make_path("_snapshot", repository, snapshot), params=params
+            "GET",
+            _make_path("_snapshot", repository, snapshot),
+            params=params,
+            headers=headers,
         )
 
     @query_params("master_timeout", "timeout")
-    def delete_repository(self, repository, params=None):
+    def delete_repository(self, repository, params=None, headers=None):
         """
         Deletes a repository.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -85,11 +92,14 @@ class SnapshotClient(NamespacedClient):
             raise ValueError("Empty value passed for a required argument 'repository'.")
 
         return self.transport.perform_request(
-            "DELETE", _make_path("_snapshot", repository), params=params
+            "DELETE",
+            _make_path("_snapshot", repository),
+            params=params,
+            headers=headers,
         )
 
     @query_params("local", "master_timeout")
-    def get_repository(self, repository=None, params=None):
+    def get_repository(self, repository=None, params=None, headers=None):
         """
         Returns information about a repository.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -101,11 +111,11 @@ class SnapshotClient(NamespacedClient):
             to master node
         """
         return self.transport.perform_request(
-            "GET", _make_path("_snapshot", repository), params=params
+            "GET", _make_path("_snapshot", repository), params=params, headers=headers
         )
 
     @query_params("master_timeout", "timeout", "verify")
-    def create_repository(self, repository, body, params=None):
+    def create_repository(self, repository, body, params=None, headers=None):
         """
         Creates a repository.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -122,11 +132,15 @@ class SnapshotClient(NamespacedClient):
                 raise ValueError("Empty value passed for a required argument.")
 
         return self.transport.perform_request(
-            "PUT", _make_path("_snapshot", repository), params=params, body=body
+            "PUT",
+            _make_path("_snapshot", repository),
+            params=params,
+            headers=headers,
+            body=body,
         )
 
     @query_params("master_timeout", "wait_for_completion")
-    def restore(self, repository, snapshot, body=None, params=None):
+    def restore(self, repository, snapshot, body=None, params=None, headers=None):
         """
         Restores a snapshot.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -147,11 +161,12 @@ class SnapshotClient(NamespacedClient):
             "POST",
             _make_path("_snapshot", repository, snapshot, "_restore"),
             params=params,
+            headers=headers,
             body=body,
         )
 
     @query_params("ignore_unavailable", "master_timeout")
-    def status(self, repository=None, snapshot=None, params=None):
+    def status(self, repository=None, snapshot=None, params=None, headers=None):
         """
         Returns information about the status of a snapshot.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -168,10 +183,11 @@ class SnapshotClient(NamespacedClient):
             "GET",
             _make_path("_snapshot", repository, snapshot, "_status"),
             params=params,
+            headers=headers,
         )
 
     @query_params("master_timeout", "timeout")
-    def verify_repository(self, repository, params=None):
+    def verify_repository(self, repository, params=None, headers=None):
         """
         Verifies a repository.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -185,11 +201,14 @@ class SnapshotClient(NamespacedClient):
             raise ValueError("Empty value passed for a required argument 'repository'.")
 
         return self.transport.perform_request(
-            "POST", _make_path("_snapshot", repository, "_verify"), params=params
+            "POST",
+            _make_path("_snapshot", repository, "_verify"),
+            params=params,
+            headers=headers,
         )
 
     @query_params("master_timeout", "timeout")
-    def cleanup_repository(self, repository, params=None):
+    def cleanup_repository(self, repository, params=None, headers=None):
         """
         Removes stale data from repository.
         `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-snapshots.html>`_
@@ -203,5 +222,8 @@ class SnapshotClient(NamespacedClient):
             raise ValueError("Empty value passed for a required argument 'repository'.")
 
         return self.transport.perform_request(
-            "POST", _make_path("_snapshot", repository, "_cleanup"), params=params
+            "POST",
+            _make_path("_snapshot", repository, "_cleanup"),
+            params=params,
+            headers=headers,
         )
